<div class="chatlog__embed-field  chatlog__embed-field--inline ">
        <div class="chatlog__embed-field-name"><span class="markdown preserve-whitespace">{{FIELD_NAME}}</span></div>
        <div class="chatlog__embed-field-value"><span class="markdown preserve-whitespace">{{FIELD_VALUE}}</span></div>
</div>

