<div class="chatlog__reference">
        <img class="chatlog__reference-avatar" src="{{AVATAR_URL}}" alt="Avatar" loading="lazy">
        {{BOT_TAG}}
        <div class="chatlog__reference-author" style="{{USER_COLOUR}}" title="{{NAME_TAG}}">{{NAME}}</div>
        <div class="chatlog__reference-content">
            <span class="chatlog__reference-link" onclick="scrollToMessage(event, '{{MESSAGE_ID}}')">
                    <em>{{CONTENT}}</em> {{ATTACHMENT_ICON}}
            </span>
        </div>
</div>