<div class=chatlog__reaction>
    <img class="emoji emoji--small" src="https://cdn.discordapp.com/emojis/{{EMOJI}}.{{EMOJI_FILE}}">
    <span class="chatlog__reaction-count">{{EMOJI_COUNT}}</span>
</div>