<div class=chatlog__attachment>
    <div class="" onclick="">
        <div class="">
            <div class="chatlog__attachment-container">
                <img src="{{ATTACH_ICON}}" class="chatlog__attachment-icon">
                <div class="chatlog__attachment-filename">
                    <a href={{ATTACH_URL}}>{{ATTACH_FILE}}</a>
                </div>
                <div class="chatlog__attachment-filesize">
                    {{ATTACH_BYTES}}
                </div>
            </div>
        </div>
    </div>
</div>