<div class=chatlog__embed>
    <div class=chatlog__embed-color-pill style=background-color:rgba({{EMBED_R}},{{EMBED_G}},{{EMBED_B}},1)></div>
    <div class=chatlog__embed-content-container>
        <div class=chatlog__embed-content>
            <div class=chatlog__embed-text>
                {{EMBED_AUTHOR}}
                {{EMBED_TITLE}}

                {{EMBED_DESC}}
                <div class="chatlog__embed-fields">
                    {{EMBED_FIELDS}}
                </div>


                {{EMBED_IMAGE}}

            </div>



            {{EMBED_THUMBNAIL}}

        </div>

        {{EMBED_FOOTER}}

    </div>
</div>

