<div class="chatlog__component-dropdown {{DISABLED}}">
  <button onclick="showDropdown({{ID}})" class="dropdownButton" id="dropdownButton{{ID}}">{{PLACEHOLDER}} {{ICON}}</button>
  <div id="dropdownMenuContent{{ID}}">
    {{CONTENT}}
  </div>
</div>