<div class="chatlog__component-button {{DISABLED}}" style=background-color:{{STYLE}}>
    <a href="{{URL}}" style=text-decoration:none>
    <span class="chatlog__button-label">{{EMOJI}}{{LABEL}}{{ICON}}</span>
    </a>
</div>